@@ -121,6 +121,12 @@ bool File::Open(const wchar *Name,uint Mode)
     hNewFile=fdopen(handle,UpdateMode ? UPDATEBINARY:READBINARY);
 #endif
   }
+#ifdef _ANDROID
+  // If we open an existing file in r&w mode and external card is read-only
+  // for usual file API.
+  if (hNewFile==FILE_BAD_HANDLE && UpdateMode && errno!=ENOENT)
+    hNewFile=JniOpenFile(Name);
+#endif
   if (hNewFile==FILE_BAD_HANDLE && errno==ENOENT)
     ErrorType=FILE_NOTFOUND;
 #endif
@@ -193,6 +199,8 @@ bool File::Create(const wchar *Name,uint Mode)
 #ifdef _ANDROID
   if (hFile==FILE_BAD_HANDLE)
     hFile=JniCreateFile(Name); // If external card is read-only for usual file API.
+  if (hFile!=FILE_BAD_HANDLE)
+    JniFileNotify(Name,false);
 #endif
 #else
   hFile=fopen(NameA,WriteMode ? WRITEBINARY:CREATEBINARY);
@@ -281,10 +289,10 @@ bool File::Rename(const wchar *NewName)
 }
 
 
-void File::Write(const void *Data,size_t Size)
+bool File::Write(const void *Data,size_t Size)
 {
   if (Size==0)
-    return;
+    return true;
   if (HandleType==FILE_HANDLESTD)
   {
 #ifdef _WIN_ALL
@@ -301,9 +309,10 @@ void File::Write(const void *Data,size_t Size)
     }
 #endif
   }
+  bool Success;
   while (1)
   {
-    bool Success=false;
+    Success=false;
 #ifdef _WIN_ALL
     DWORD Written=0;
     if (HandleType!=FILE_HANDLENORMAL)
@@ -352,6 +361,7 @@ void File::Write(const void *Data,size_t Size)
     break;
   }
   LastWrite=true;
+  return Success; // It can return false only if AllowExceptions is disabled.
 }
 
 
@@ -565,7 +575,20 @@ bool File::Truncate()
 #ifdef _WIN_ALL
   return SetEndOfFile(hFile)==TRUE;
 #else
-  return false;
+  return ftruncate(GetFD(),(off_t)Tell())==0;
+#endif
+}
+
+
+void File::Flush()
+{
+#ifdef _WIN_ALL
+  FlushFileBuffers(hFile);
+#else
+#ifndef FILE_USE_OPEN
+  fflush(hFile);
+#endif
+  fsync(GetFD());
 #endif
 }
 
